/**
 *	he_echod  --  hecom server test program
 *		Copyright (C) Kazuro Furukawa, 1988-1993
 *		Kazuro Furukawa, KEK, Japan (furukawa@kek.jp)
 *
 *	K.Furukawa  Sep.15.1993. hecomserver based on s2_rechod v1.2.1
 *	Modifications
 *	K.Furukawa  Sep.17.1993. name change from hecomserver to he_echod
 *
 *      This program receives messages from hecomclient program and echos back
 *      the the same message to client program.  This will be the typical
 *	example for the hecom server-client model.
**/

/**
 *	s2_rechod  --  remote echo server program
 *		Copyright (C) Kazuro Furukawa, 1988-1993
 *		Kazuro Furukawa, KEK, Japan (furukawa@kek.jp)
 *
 *	K.Furukawa  Nov.9.1988. (rwrited)
 *	K.F  Apr.13.1989.  converted from rwrited (rechod)
 *	K.F  Jun.17.1992.  rechod v3.4.2
 *	K.F  Oct.2.1992.   s2_rechod using s2_xxx routines
 *	Modifications
 *	K.F  Nov.6.1992.   single server exit fix
 *	K.F  Nov.26.1992.  use getopt routine
 *
 *      This program receives messages from s2_recho program and echos back
 *      the the same message to recho program.  This will be the typical
 *	example for the TCP|UDP/IP server-client model.
**/

char	what[] = 
"@(#)he_echod.c v1.1.0 hecom echo server, K.Furukawa, Nov.1988-Sep.1993";

#include <stdio.h>

#define  PORTNUMB	11061			/* default service port */
#define  BUFFERSIZE	16384			/* communication buffer size */

#define  OPT		3			/* error message option */

char	*nm;
char	buf[BUFFERSIZE];			/* communication buffer */
int	portnum;				/* port number */

/**
 *  main
 *	Main prepares communication line.
**/
void
 main(argc, argv)
int	argc;
char	*argv[];
{
    void argument();
    void errormsg();
    void server();
    int	sl;					/* socket desctiptor */

    nm = argv[0];				/* program name */
    portnum = 0;				/* port number */
    
    argument(argc, argv);			/* command line argument */
    
    if ( !portnum )  {				/* port number not specified */
	portnum = PORTNUMB;			/* default port */
    }

    fclose( stdin );				/* close control terminal */
    sl = he_open( portnum, BUFFERSIZE, OPT );	/* open communication */
    he_init( OPT );				/* reset communication */
    if ( sl<0 )  exit( 1 );

    server( sl );				/* server */

}

/**
 *  server
 *	server waits for message from clients.  If a message arrives,
 *	it echoes the message back to the sender.
**/
void
 server( sl )
int sl;						/* listen socket */
{
    int	cc;

    while( ( cc = he_accept( buf, BUFFERSIZE, OPT ) ) >= 0 )  {
						/* read message */
	if( he_reply( buf, cc, OPT ) < cc ) {	/* echo back*/
	    exit( 1 );
	}

    }						/* repeat forever */
    exit( 1 );
}

/*
 *  errormsg
 *	print error message
 */
void
 errormsg( msg )
char * msg;
{
#ifndef OSK
	perror( nm );
#else
	extern int errno;
	prerr(0,errno);
	fprintf(stderr,"%s\n", nm);
#endif
	fprintf( stderr, "%s: %s\n", nm, msg );
}

/**
 *  argument
 *	Argument handles command line argument, such as hostname or options.
**/
void
 argument(argc, argv)
int	argc;
char	*argv[];
{
	int c;
	extern int optind, opterr;
	extern char *optarg;
	opterr = 0;			/* disable informational message */

	while ((c = getopt (argc, argv, "p:P:hH")) != EOF)
	    switch ( c )  {
	    case 'p':
	    case 'P':				/* port number */
		portnum = atoi( optarg );
		break;
	    case '?':
	    case 'h':
	    case 'H':				/* help */
	    default:
		goto usage;			/* print out usage */
	    }
	if ( argc == optind )  return;

usage:
	fprintf(stderr, "%s\n", what );
	fprintf(stderr, "Usage:  he_echod  <option>\n" );
	fprintf(stderr, " option:\n" );
	fprintf(stderr, "          -pX  use service port number X\n" );
	fprintf(stderr, "          -h   help message\n" );
	fprintf(stderr, " example: he_echod \n" );
	fprintf(stderr, "          he_echod -p2000\n" );
	exit( 1 );
}
