.\"
.\"	@(#)hecomsub.man	1.0 (KEK)
.\"
.\"	Kazuro Furukawa, Kek.
.\"
.PU
.TH HECOMSUB 3 "July 8, 1993"
.SH NAME
hecomsub \- inter process communication routines
.SH DESCRIPTION

.\"	Below is the file hecomsub.doc
.EX 1
hecomsub.doc
	Kazuro Furukawa (furukawa@kek.jp)
	Jul.8.1993., Sep.15,25.1993.

@(#)hecomsub.doc v1.0.4 K.Furukawa, Jul.1993-Sep.1993

File hecomsub.c containes he_xxx functions for message communication
routines.  They use fs_xxx semaphore routines in semsub.c and cm_xxx 
shared-memory routines in shmsub.c, which are described in semsub.doc
and shmsub.doc respectively.
These routines are based on swevent programs. This package is available 
at ~furukawa/work/shmsem/shmsem.tar.Z or 
olive.kek.jp:~ftp/pub/unix/shmsem.tar.Z.
The library name is libhe.a (libhe.l for OS9), which includes hecomsub,
semsub and shmsub routines.

This package acts like UDP communication system, but is reliable. 
It forms multi clients, single server communication system. 
However, you must follow the sequence, send-accept-reply-receive. 

The file hecomsub.c defines he_open, he_send, he_accept, he_reply, 
he_receive and he_close routines.

These routines run on Sony/News, Sun, VAX/Ultrix, DecStation/Ultrix, 
Hpux, NEC/SXA, Apple/AUX, LynxOS, and OS9/68K.
They don't run on NeXT, Mitsubishi/Umx, MSDOS and MacOS/MacTCP.

BASIC USAGE:
 The basic usage below describes resource locking between two processes.
  process A (client):
   he_open( 123, 1024, 1 );			/* open communication
   he_send( buffer, size, 1 );			/* send request message
   ...
   size = he_receive( buffer, size, 1 );	/* receive reply message
   ...
   he_close( 1 );				/* close channel
  process B (server):
   he_open( 123, 1024, 1 );			/* open communication
   size = he_accept( buffer, size, 1 );		/* accept request
   ...
   he_reply( buffer, size, 1 );			/* send reply message
   ...
   he_close( 1 );				/* close channel

EXAMPLE PROGRAMS:
 he_echo  : client test program
 he_echod : server test program

BUGS:
 Currently he_remove is not implemented.  

REFERENCE MANUAL:
 Reference manual below is generated from source file with the command
 "comment hecomsub.c".

/**
 *	hecomsub  --  communication subroutines
 *		Kazuro Furukawa, KEK, Japan (furukawa@kek.jp)
 *		(if you modified this code, please inform furukawa)
 *
 *	K.Furukawa  Mar.24.1992.(2)  comsub
 *	K.F         Aug.29.1988.     swmodule for os9
 *	K.F         Jul.8.1993.      hecomsub
 *	Modifications
 *	K.F         Sep.15.1993.     no warning message in he_open
 *	K.F         Sep.15.1993.     accept multiple clients
 *	K.F         Sep.15.1993.     fix bug in semaphore handling
 *	K.F         Sep.18.1993.     change header size
 *
 *	This module defines communication routines using semaphore
 *	and shared memory.
 *	 int he_open( portnum, buffersize, opt )
 *	 int he_send( buffer, size, opt )
 *	 int he_accept( buffer, size, opt )
 *	 int he_reply( buffer, size, opt )
 *	 int he_receive( buffer, size, opt )
 *	 int he_close( opt )
 *	 int he_init( opt )
 *	If option is 1, error message is printed on to stderr 
 *	when error occured.
 *	If option is 3, error and information message is printed on to stderr 
 *	when error occured.
 *	Error codes errno returned from the system subroutines are preserved 
 *	on return to the caller.
 *	It uses one shared memory block and three semaphores internally.
 */

static char hecomsub_what[] = 
"@(#)hecomsub.c v1.4.1 communication subroutines, K.Furukawa, Sep.1993"; /*
**/

/**
 *  he_open
 *	he_open prepares communication
 *
 *    int he_open( portnum, buffersize, option )
 *
 *	portnum is the communication port number, which is used as
 *	shared memory and semaphore key numbers
 *	buffersize is the maximum communication buffer size
 *	if option is 1 error message is sent to stderr
 *	if option is 3 error/information message is sent to stderr
 *
 *    return:	positive/0	success
 *		<0		failure
 */
int 
 he_open( portnum, size, option )
int portnum;			/* port number */
int size;			/* communication buffer size */
int option;			/* option 1: error, 2: info. message */
/*
**/

/**
 *  he_init
 *	he_init initialize communication internals
 *
 *    int he_init( option )
 *
 *	if option is 1 error message is sent to stderr
 *	if option is 3 error/information message is sent to stderr
 *
 *    return:	positive/0	success
 *		<0		failure
 */
int 
 he_init( option )
int option;			/* option 1: error, 2: info. message */
/*
**/

/**
 *  he_send
 *	he_send sends message to server
 *
 *    int he_send( buffer, size, option )
 *
 *	if option is 1 error message is sent to stderr
 *	if option is 3 error/information message is sent to stderr
 *
 *    return:	positive/0	success, sent buffer size
 *		<0		failure
 */
int 
 he_send( buffer, size, option )
void * buffer;			/* message buffer */
int size;			/* communication buffer size */
int option;			/* option 1: error, 2: info. message */
/*
**/

/**
 *  he_accept
 *	he_accept receives message from client
 *
 *    int he_accept( buffer, size, option )
 *
 *	if option is 1 error message is sent to stderr
 *	if option is 3 error/information message is sent to stderr
 *
 *    return:	positive/0	success, received buffer size
 *		<0		failure
 */
int 
 he_accept( buffer, size, option )
void * buffer;			/* message buffer */
int size;			/* communication buffer size */
int option;			/* option 1: error, 2: info. message */
/*
**/

/**
 *  he_reply
 *	he_reply sends back message to client
 *
 *    int he_reply( buffer, size, option )
 *
 *	if option is 1 error message is sent to stderr
 *	if option is 3 error/information message is sent to stderr
 *
 *    return:	positive/0	success, sent buffer size
 *		<0		failure
 */
int 
 he_reply( buffer, size, option )
void * buffer;			/* message buffer */
int size;			/* communication buffer size */
int option;			/* option 1: error, 2: info. message */
/*
**/

/**
 *  he_receive
 *	he_receive receives message from server
 *
 *    int he_receive( buffer, size, option )
 *
 *	if option is 1 error message is sent to stderr
 *	if option is 3 error/information message is sent to stderr
 *
 *    return:	positive/0	success, received buffer size
 *		<0		failure
 */
int 
 he_receive( buffer, size, option )
void * buffer;			/* message buffer */
int size;			/* communication buffer size */
int option;			/* option 1: error, 2: info. message */
/*
**/

/**
 *  he_close
 *	he_close terminates communication
 *	this version does not remove shared memory and semaphore
 *
 *    int he_close( option )
 *
 *	if option is 1 error message is sent to stderr
 *	if option is 3 error/information message is sent to stderr
 *
 *    return:	positive/0	success
 *		<0		failure
 */
int 
 he_close( option )
int option;			/* option 1: error, 2: info. message */
/*
**/

